/*---------------------------------------------------------------------------------------------------*\
|   Program name: Macro to merge back patids with no claim in the CCI period (CCI score=0) and to     |
|                  assign grouping formats based on the CCI value                                     |
|                                                                                                     |
|   Date: 08/06/2012                                                                                  |
|   Version: Beta 1                                                                                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program will use patient from INDEXFILE to add back patient who had no claim in the          |
|   CCI calculation period.  Also, this macro will assing user-defined groups of CCI formats to be    |
|   used for stratifying patients based on their score group.                                         |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  - One SAS dataset containing out of the ms_cci macro                               |
|   Program outputs:                                                                                  |
|                  - SAS dataset (.SAS7BDAT) containing the same number of records than in INDEXFILE  |
|                    with the CCI variables computed for each of them                                 |
|																									  |
|   Macro variable definitions:  																	  |
|	DSIN = Name of the SAS dataset containing the calculated CCI variables                            |
|  	INDEXFILE = The name of the sas file containing the Patid and reference date to compute the CCI   |
|  	OUTFILE = The final formatted SAS dataset                                                       |
|  	CCIGROUP = The grouping that the user wishes to apply.  Grou need to be separate by a space and   |
|              use "+" if the last group is open ended                                                |
|   KEEPALLDUM = set equal to one if the user wants to keep all individual CCI dummies ( P and I)     |
|                  in the final file                                                                  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
*This macro needs the ms_wordcount tool;
*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;    

%macro ms_cci_post(DSIN=,INDEXFILE=,OUTFILE=,CCIGROUP=,KEEPALLDUM=);	

/*****************************************************************/
/* Merge back data from members without any C4 and 09 claim      */
/* within the CCI calculation window and assign zero scores.     */
/* Also, assing CCI grouping for stratification                  */
/*****************************************************************/

	%MS_WORDCOUNT(TXTVECTOR=&CCIGROUP., WORDCOUNT=NUMGRP);

	%let ALLVAR=;
	data _null_;
		format FMTSTR $300.;
		if &KEEPALLDUM.=1 then call symput("ALLVAR","_ALL_");
		if &NUMGRP=0. then FMTSTR='0-high = "0+"';
		else do i=1 to &NUMGRP.;
			word=tranwrd(scan("&CCIGROUP.",i,' '), "+", "-high")||' = "'|| scan("&CCIGROUP.",i,' ')||'" ';
			FMTSTR=strip(FMTSTR)||" "||trim(left(word));
			output;
		end;
		call symput("FMTSTR",FMTSTR);
	run;
	%put &FMTSTR.;

	PROC FORMAT;
	   VALUE CCIGRP
		&FMTSTR.
	;
	run;

	data &OUTFILE.;
	format PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM CCIGRP.;
	retain patid PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM;		
	merge &INDEXFILE.(in=a keep=PatId) &DSIN.(rename=PCHRLSON=PCHRLSON_NUM 
											  rename=CHRLSON=CHRLSON_NUM
											  rename=XCHRLSON=XCHRLSON_NUM);
	by PatId;	
	if a;

	if PCHRLSON_NUM=. then PCHRLSON_NUM=0;
	if CHRLSON_NUM=. then CHRLSON_NUM=0;
	if XCHRLSON_NUM=. then XCHRLSON_NUM=0;

	keep &ALLVAR. Patid PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM;
	run;

	data &OUTFILE.;
	retain patid PCHRLSON CHRLSON XCHRLSON PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM;
	format PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM best.;
	set &OUTFILE.;
	PCHRLSON=put(PCHRLSON_NUM, CCIGRP.);
	CHRLSON=put(CHRLSON_NUM, CCIGRP.);
	XCHRLSON=put(XCHRLSON_NUM, CCIGRP.);	
	
	label PCHRLSON=Prior Charlson comorbidity score;
	label CHRLSON=Prior+Index Charlson comorbidity score;
	label XCHRLSON=Index Charlson comorbidity score;
	label PCHRLSON_NUM=Prior Charlson comorbidity individual score;
	label CHRLSON_NUM=Prior+Index Charlson comorbidity individual score;
	label XCHRLSON_NUM=Index Charlson comorbidity individual score;
	run;

	proc freq data=&OUTFILE.;
	tables PCHRLSON_NUM CHRLSON_NUM XCHRLSON_NUM;	
	run;

	proc freq data=&OUTFILE.;
	tables PCHRLSON CHRLSON XCHRLSON;
	run;
	
%mend ms_cci_post;


