/*---------------------------------------------------------------------------------------------------*\
|   Program name: Data subset creation tool.                     									  |														  
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program creates a subset of a dataset containing patient level data.						  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS dataset with patient level data							    			      |
|                  -SAS dataset with patients list for subset creation   						      |
|																									  |
|   Program outputs:                                                                                  |
|                  -SAS dataset with subset of patient level data  								      |
|                                                                                                     |
|   Macro variable definitions:  																	  |
|	INFILE		  = SAS dataset with patient-level data												  |	
|  	PATLISTINFILE = SAS dataset with list of patients							                      |	
|  	OUTFILE		  = SAS dataset output file with subset of patient level data						  |
|			      																					  |                                                   
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_FREEZEDATA(INFILE=,PATLISTINFILE=,OUTFILE=);
		
	proc sort nodupkey data=&PATLISTINFILE.(keep=PatId)
					   out=_tempPatList;
	by PatId;
	run;	
	
	data &OUTFILE.;
	set _tempPatList point = _n_;
	declare hash ht (hashexp:16, dataset:"_tempPatList");   
	ht.definekey('Patid');
	ht.definedata(ALL: 'YES');
	ht.definedone();

	do until(eof1);
		set &INFILE. end=eof1;
		if ht.find()=0 then output;
	end;
	stop;
	run;

	proc datasets library = work nolist nowarn nodetails;
	delete _tempPatList;
	quit;

%mend MS_FREEZEDATA;
