/*---------------------------------------------------------------------------------------------------*\
|   Program name: Creation of a subset of a dataset patients list.									  |														  
|                                                                                                     |
|   Date: 06/29/12                                                                                    |
|   Version: Beta 1                                                                                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program creates a subset of a dataset containing patients data.							  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with patients data							    			      |
|                  -SAS data file with patients list for subset	creation   						      |
|																									  |
|   Program outputs:                                                                                  |
|                  -SAS data file patients data subset												  |
|                                                                                                     |
|   Macro variable definitions:  																	  |
|	INFILE		  = SAS data file with patients data												  |	
|  	PATLISTINFILE = SAS data file with patients list for subset creation							  |	
|  	OUTFILE		  = SAS data file patients data subset												  |
|                                                                                                     |
|   Programming Notes:                                                                                |
|			      																					  |                                                   
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_LOCKDATA(INFILE=,PATLISTINFILE=,OUTFILE=);
		
	proc sort nodupkey data=&PATLISTINFILE.(keep=PatId)
					   out=_tempPatList;
	by PatId;
	run;	
	
	data &OUTFILE.;
	set _tempPatList point = _n_;
	declare hash ht (hashexp:16, dataset:"_tempPatList");   
	ht.definekey('Patid');
	ht.definedata(ALL: 'YES');
	ht.definedone();

	do until(eof1);
		set &INFILE. end=eof1;
		if ht.find()=0 then output;
	end;
	stop;
	run;

	proc datasets library = work nolist nowarn nodetails;
	delete _tempPatList;
	quit;

%mend MS_LOCKDATA;
