/*---------------------------------------------------------------------------------------------------*\
|   Program name: Transfer unique values of a dataset column into a macro variable.                   |														  
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program transfers unique values of a dataset column into a macro variable.                   |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with values to transfer into a macro variable                       |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -Global macro variable with values of the specified input file column              |
|																									  |
|   Macro variable definitions:  																	  |
|	INFILE		= SAS dataset with values to transfer into the macro variable                         |
|  	VARCOLUMN	= Name of the column to transfer into the macro variable                              |
|  	PREFIX		= Characters to be added before each value transferred into the macro variable        |
|  	SUFFIX		= Characters to be added after each value transferred into the macro variable         |
|  	SEPARATOR	= Separator for each value transferred into the macro variable				          |
|  	OUTVAR		= Output global macro variable                                                        |
|			      																					  |                                                  
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_MAKEMACVAR(INFILE=,VARCOLUMN=,PREFIX=,SUFFIX=,SEPARATOR=,OUTVAR=);


	%global &OUTVAR.;

	%if %sysfunc(exist(&INFILE.))=1 %then %do;

		*Get the length of the column to transfer to create a new one with prefix and suffix;
		proc contents data=&INFILE. noprint out=_tilt_ ;
    	data _null_;
    	set _tilt_ ;
    	if trim(upcase(NAME))=trim(upcase("&VARCOLUMN.")) then do;
			call symput('columnlength',trim(left(put(length,best.))));
		end;
    	run;		
		%let columnlength = %eval(%length(&PREFIX) + &columnlength. + %length(&SUFFIX));
		%put &columnlength.;

		data _null_;
		call symput("&OUTVAR.","");
		run;	
		data _temp(keep=new_&VARCOLUMN.);
		set  &INFILE.;
		format new_&VARCOLUMN. $&columnlength..;
		new_&VARCOLUMN. = &PREFIX.||trim(left(&VARCOLUMN.))||&SUFFIX.;
		run;
		proc sql noprint;
		select unique new_&VARCOLUMN.
		  into :&OUTVAR. separated by &SEPARATOR. 
		  from _temp;
		quit;
		%put &&&OUTVAR.;

		proc datasets library = work nolist;
		delete _temp _tilt_;
		quit;
	%end;

%mend MS_MAKEMACVAR;
