/*---------------------------------------------------------------------------------------------------*\
|   Program name: Claims or episodes within enrollment/eligibility Tool.     						  |														  
|                                                                                                     |
|   Date: 11/28/2012                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   This program removes claims or episodes that are outside a period of                              |
|   enrollment/eligibility. The enrollment/eligibility information can be contained in and called     |
|   from an external dataset or it can be specified as fixed dates in an input macro variable.	      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with claims or episodes to check for enrollment/eligibility         |
|                  -SAS data file with enrollment/eligibility information 						      |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file with claims or episodes within enrollment/eligibility only		  |
|																									  |
|   Macro variable definitions:  																	  |
|	INFILE		= SAS dataset with claims or episode data					                          |
| 	STARTDT		= Name of the claim/episode start date variable										  |
|  	ENDDT		= Name of the claim/episode end date variable										  |
|  	ELIGINFILE	= SAS dataset with enrollment/eligibility data										  |
|  	ELIGSTARTDT	= Name of the enrollment/eligibility start date variable or a fixed date			  |
|  	ELIGENDDT	= Name of the enrollment/eligibility end date variable or a fixed date				  |
|  	OUTFILE		= SAS dataset with claims or episodes within enrollment/eligibility only 			  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

%macro MS_WITHINELIG(INFILE=,STARTDT=,ENDDT=,ELIGINFILE=,ELIGSTARTDT=,ELIGENDDT=,OUTFILE=);
		
	*Check if eligibility start date is a date or a variable in a dataset;
	%if %index(&ELIGSTARTDT.,/) %then %do;
		data _null_;
		call Symput('eligstartdateval',put(input("&ELIGSTARTDT.",mmddyy10.),best12.));
		run;
		%put ELIG. START DATE CONVERTED TO NUMERIC: &eligstartdateval.;
	%end;
	%else %do;
		%let eligstartdateval=elig.&ELIGSTARTDT.;
	%end;

	*Check if eligibility end date is a date or a variable in a dataset;
	%if %index(&ELIGENDDT.,/) %then %do;
		data _null_;
		call Symput('eligenddateval',put(input("&ELIGENDDT.",mmddyy10.),best12.));
		run;

		%put ELIG. END DATE CONVERTED TO NUMERIC: &eligenddateval.;
	%end;
	%else %do;
		%let eligenddateval=elig.&ELIGENDDT.;
	%end;

	*Create output file;
	%if %str("&ELIGINFILE.") ne %str("") %then %do;
		proc sql noprint;
		create table &OUTFILE. as 
			select in.*,
                   &eligstartdateval.,
                   &eligenddateval.
			from &INFILE. as in,
				 &ELIGINFILE. as elig
		    where in.PatId=elig.PatId and &eligstartdateval. <= in.&ENDDT. and in.&STARTDT. <= &eligenddateval.;   	
		quit;

	%end;
	%else %do;
		proc sql noprint;
		create table &OUTFILE. as 
			select in.*,
				   &eligstartdateval. as  EligStartDt format=mmddyy10.,
				   &eligenddateval. as EligEndDt format=mmddyy10.
			from &INFILE. as in
		    where &eligstartdateval. <= in.&ENDDT. and in.&STARTDT. <= &eligenddateval.;   	
		quit;
	%end;


%mend MS_WITHINELIG;
