****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: 04_table_creator.sas  
*
* Created (mm/dd/yy): 11/01/2013
* Last modified: 11/01/2013
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Mini-Sentinel : Propensity Score Matching PROMPT (Prospective Routine Observational Monitoring Program Tool) 
*
*	 The Initial Developer of the Original Code is the 							  
*	 Division of Pharmacoepidemiology and Pharmacoeconomics, 					  
*	 Department of Medicine, Brigham and Women's Hospital and 					  
*	 Harvard Medical School					  									  
*																				  
*	 Module Leads: Joshua J Gagne, Shirley V Wang, Sebastian Schneeweiss			  
*	 Programmer:   Shirley V Wang												  
*	 Uses Pharmacoepidemiology Toolbox macros, created by Jeremy Rassen	
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;

/**************************************************************
 CREATE TABLES & PROPENSITY DISTRIBUTION FIGURES
***************************************************************/

%macro matchtables();

options mprint mlogic;

proc printto log="&MSOC.&DPID.&SITEID._&REQUESTID._table1_&look..log" new;
run;
	
/******************************************************
   GET START TIME
 ******************************************************/

        data _NULL_;
        temp=DATETIME();
        call symput('START',temp);
        call symput('STARTDATE',put(datepart(temp),date9.));
        call symput('STARTTIME',put(timepart(temp),time4.));
        run;

/********************************************************
   Directories to include in program
 ********************************************************/

* Set the directory for template table layout file (xls file);
%let aptab = %sysfunc(trim(&infolder.&templatetable..xls));

/** Specify name of input and output datasets for table creation       **/
/** Set the location of the output rtf file in the specified directory **/
%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK)) = 0 %then %do;
	%put error: Input dataset does not exist;
	%abort;
%end;
%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._matched_&LOOK)) = 1 %then %do;
	%let input = &DPID.&SITEID._&REQUESTID._matched_&LOOK;
	%let taboc = %sysfunc(trim(&msoc.&DPID.&SITEID._&REQUESTID._matched_tables_&Look..xls));
	%let rtff = %sysfunc(trim(&msoc.&DPID.&SITEID._&REQUESTID._matched_PSDistribution_&look..rtf));
%end;

/* 	Skip matched tables if there are no matches 
	from any of sequential monitoring periods */

	%let skip = ;
	PROC SQL;
	create table exposure_time as
	select count(distinct studyclass) as c
	from dplocal.&input
	group by time;
	select max(c) into :numexposure
	from exposure_time;
	QUIT;

	%if &numexposure <2 %then %let skip = 1;

* Set the list of dichotomous and continuous variables;
%if &ITT = %str() %then %let a_type = ASTREATED;
%if &ITT ne %str( ) %then %let a_type = ITT;
 
%let covar = ;
%let covmed = ;
%let covproc =;
%let util =;
%let agecat =;

PROC CONTENTS DATA = DPLOCAL.&INPUT
OUT=VAR_NAMES(KEEP = NAME) NOPRINT;
RUN;

DATA var_names;
	SET var_names;
	u_name = UPCASE(name);
RUN;

PROC SQL; 

select name 
into :covar separated by ' '
from var_names
where index(u_name, "COVAR") = 1 and index(name, "TIME") = 0;

select name 
into :covproc separated by ' '
from var_names
where index(u_name, "COVPROC") = 1 and index(name, "TIME") = 0;

select name 
into :covmed separated by ' '
from var_names
where index(u_name, "COVMED") = 1 and index(name, "TIME") = 0;

select name 
into :util separated by ' '
from var_names
where index(u_name, "NUM") = 1;

select distinct(compress("AGE"||compress(age_cat, "<-y"))) 
into :agecat separated by ' '
from dplocal.&input;

QUIT;

%put &covar;
%put &covmed;
%put &covproc;
%put &util;
%put &agecat;

%let dvarlst = 	EVENT_&a_type &agecat female male unknown &covar &covmed &covproc;
%let cvarlst = 	FOLLOWUPTIME_&a_type AGE COMORBIDSCORE &util;

* Set library name;

LIBNAME tables "&msoc.";

/* initialize the table code */
%table_Init(tables);

%macro resultsout(data_input = , tabledvars = ,tablecvars = );

	proc sql;
	select count(distinct age_cat) 
	into :totagecat 
	from dplocal.&input;
	select distinct age_cat
	into :age1-:age%eval(&totagecat)
	from dplocal.&input;
	quit;
	%put &totagecat;
	%put &age1;

%macro w();
  data datain2; 
  set &data_input; 
  NPAT = 1;

  FEMALE = (SEX = 'F');
  MALE =  (SEX = 'M');
  UNKNOWN = (SEX in('U', 'A'));

  %do i = 1 %to %eval(&totagecat);
  %scan(&agecat,%eval(&i)) =(Age_Cat = "&&Age&i");
  %end;

  PRI_TX = (StudyClass = "&grp1");
  PRI_REF = (StudyClass = "&grp2");

  %if &skip = %str() %then %do;
  MATCHEDPRED_CAL01 = (PREDEFINEDPSMATCHID1_1 ne '');
  MATCHEDPRED_CAL025 = (PREDEFINEDPSMATCHID1_2 ne '');
  MATCHEDPRED_CAL05 = (PREDEFINEDPSMATCHID1_3 ne '');
  MATCHEDHDPS_CAL01 = (HDPSPREDEFINEDMATCHID1_1 ne '');
  MATCHEDHDPS_CAL025 = (HDPSPREDEFINEDMATCHID1_2 ne '');
  MATCHEDHDPS_CAL05 = (HDPSPREDEFINEDMATCHID1_3 ne '');
  MATCHEDHDOnly_CAL01 = (HDPSONLYMATCHID1_1 ne '');
  MATCHEDHDOnly_CAL025 = (HDPSONLYMATCHID1_2 ne '');
  MATCHEDHDOnly_CAL05 = (HDPSONLYMATCHID1_3 ne '');
  %end;
  
  %if &skip = 1 %then %do;
  MATCHEDPRED_CAL01 = 0;
  MATCHEDPRED_CAL025 = 0;
  MATCHEDPRED_CAL05 = 0;
  MATCHEDHDPS_CAL01 = 0;
  MATCHEDHDPS_CAL025 = 0;
  MATCHEDHDPS_CAL05 = 0;
  MATCHEDHDOnly_CAL01 = 0;
  MATCHEDHDOnly_CAL025 = 0;
  MATCHEDHDOnly_CAL05 = 0;
  predefinedps = .; 
  hdpspredefined = .;
  hdpsonly = .;
  %end;

  label PRI_TX = "&grp1"
        PRI_REF = "&grp2";

  keep 	pri_tx pri_ref npat 
		&tabledvars &tablecvars age_cat sex
		predefinedps hdpspredefined hdpsonly
		MATCHEDPRED_CAL01 MATCHEDPRED_CAL025 MATCHEDPRED_CAL05 
		MATCHEDHDPS_CAL01 MATCHEDHDPS_CAL025 MATCHEDHDPS_CAL05 
		MATCHEDHDOnly_CAL01 MATCHEDHDOnly_CAL025 MATCHEDHDOnly_CAL05 ;	
run;

%mend;
%w(); 

* Get NPAT for each group;
proc sql noprint;
  select count(*) into :NPAT1 from datain2 where PRI_TX = 1;
  select count(*) into :NPAT2 from datain2 where PRI_REF = 1;
quit;
run;

%macro filltables(table = , dsname = ,dichvars = , contvars = ,  percentDec = , subgroup = );

  %local snpat1 snpat2;
  %let snpat1 = 0;
  %let snpat2 = 0;
  %if &subgroup ne %str() %then %do;
    data &subgroup;
      set &dsname;
      where &subgroup = 1;
    run;
    %let dsin = &subgroup;
    %put Subgroup Table &subgroup;
  %end;
  %else %do;
    %let dsin = &dsname;
    %put Unmatched Dataset Table;
  %end;

/* Continue if there are >0 observations in the subgroup */

	%let dsid=%sysfunc(open(&dsin));
	%let numobs=%sysfunc(attrn(&dsid,nlobs));
	%let rc=%sysfunc(close(&dsid));

	%if &numobs>0 %then %do; 

  %table_FillCell(&table, DRUGNAME, PRI_TX, %bquote(&grp1) );
  %table_FillCell(&table, DRUGNAME, PRI_REF, %bquote(&grp2) );

  proc sql noprint;
    select count(*) into :snpat1 from &dsin where PRI_TX = 1;
    select count(*) into :snpat2 from &dsin where PRI_REF = 1;
  quit;
run;
%put Number in subgroup 1 is &snpat1;
%put Number in subgroup 2 is &snpat2;

  %if &SNPAT1 eq 0 %then %do;
    %table_FillCell(&table, NPAT, PRI_TX, %bquote(- (- %)) );
  %end;
  %if &SNPAT1 ne 0 %then %do;
    %let percnpat1 = %sysfunc(putn(%sysevalf(100* &snpat1 /&npat1),8.1.));
    %table_FillCell(&table, NPAT, PRI_TX, %bquote(&snpat1 (&percnpat1 %)) );
  %end;

  %if &SNPAT2 eq 0 %then %do;
    %table_FillCell(&table, NPAT, PRI_REF, %bquote(- (- %)) );
  %end;
  %if &SNPAT2 ne 0 %then %do;
    %let percnpat2 = %sysfunc(putn(%sysevalf(100* &snpat2 /&npat2),8.1.));
    %table_FillCell(&table, NPAT, PRI_REF, %bquote(&snpat2 (&percnpat2 %)) );
  %end;

  %let totdich = %sysfunc(countw(&dichvars," "));

  %do tmp_j = 1 %to &totdich;
    %let dvar = %scan(&dichvars,&tmp_j);
    proc sql noprint;
      select sum(&dvar) into :ndich1 from &dsin where PRI_TX = 1;
      select sum(&dvar) into :ndich2 from &dsin where PRI_REF = 1;
      select count(*) into :ntot1 from &dsin where PRI_TX = 1;
      select count(*) into :ntot2 from &dsin where PRI_REF = 1;
    quit;

    %if &snpat1 eq 0 %then %do;
      %table_FillCell(&table, &dvar, PRI_TX, %bquote(- (- %)) );
    %end;
    %if &snpat1 ne 0 %then %do;
      %let pd1 = %sysfunc(putn(%sysevalf(100*&ndich1/&ntot1),8.1.));
      %let ppd1 = %sysevalf(&ndich1/&ntot1);
      %table_FillCell(&table, &dvar, PRI_TX, %bquote(&ndich1 (&pd1 %)) );
    %end;

    %if &snpat2 eq 0 %then %do;
      %table_FillCell(&table, &dvar, PRI_REF, %bquote(- (- %)) );
    %end;
    %if &snpat2 ne 0 %then %do;
      %let pd2 = %sysfunc(putn(%sysevalf(100*&ndich2/&ntot2),8.1.));
      %let ppd2 = %sysevalf(&ndich2/&ntot2);
      %table_FillCell(&table, &dvar, PRI_REF, %bquote(&ndich2 (&pd2 %)) );
    %end;

    %if &snpat1 ne 0 and &snpat2 ne 0 %then %do;
      %let pdiff = %sysfunc(putn(%sysevalf(&pd1-&pd2),8.1.));
      %table_FillCell(&table, &dvar, AD, %bquote(&pdiff));
      %if &ndich1 ne 0 and &ndich2 ne 0 and &ntot1 gt 1 and &ntot2 gt 1 %then %do;
        %let a = %sysevalf(&ppd1-&ppd2);
        %let b = %sysevalf((&ppd1+&ppd2)/2);
     	%let c = %sysevalf(%sysfunc(sqrt(&b*(1-&b))));
        %let sepdiff = %sysfunc(putn(%sysevalf((&a/&c)),8.1));
        %table_FillCell(&table, &dvar, SD, %bquote(&sepdiff));
      %end;
      %else %do;
        %table_FillCell(&table, &dvar, SD, %bquote(-));
      %end;
    %end;
    %else %do;
      %table_FillCell(&table, &dvar, AD, %bquote(-));
      %table_FillCell(&table, &dvar, SD, %bquote(-));
    %end;
  %end;

  %macro tablemean(TABLE = , VARROW = );
    %let mean1 = .;
    %let mean2 = .;
    %let std1 = .;
    %let std2 = .;
    %let adiff = .;
    %let sdiff = .;
    %let totrow = %sysfunc(countw(&VARROW," "));
    %do numrvar = 1 %to &totrow;
      %let rvar = %scan(&varrow, &numrvar); 

	  /* make macro variables without number formatting for calculating diff/std diff */
      proc sql noprint;
        select avg(&rvar) into :mean1 from &dsin where PRI_TX = 1;
        select avg(&rvar) into :mean2 from &dsin where PRI_REF = 1;
        select std(&rvar) into :std1 from &dsin where PRI_TX = 1;
        select std(&rvar) into :std2 from &dsin where PRI_REF = 1;
      quit;

      %let adiff = %sysevalf(&mean1 - &mean2);
	 
	  %if &std1>0 and &std2>0 %then %do;
      	%let sdiff = %sysevalf((&mean1 - &mean2)/(%sysfunc(sqrt((&std1*&std1 + &std2*&std2)/2))));
		%let nopoolstd = 0;
	  %end;
	  %if &std1 = 0 or &std2 = 0 %then %let nopoolstd = 1;

	 /* make macro variables with formatting */
	   proc sql noprint;
        select avg(&rvar) format 8.1 into :mean1 from &dsin where PRI_TX = 1;
        select avg(&rvar) format 8.1 into :mean2 from &dsin where PRI_REF = 1;
        select std(&rvar) format 8.1 into :std1 from &dsin where PRI_TX = 1;
        select std(&rvar) format 8.1 into :std2 from &dsin where PRI_REF = 1;
       quit;

	  %if &snpat1 eq 0 %then %do;
      %table_FillCell(&table, &rvar, PRI_TX, %bquote(- (- %)) );
      %end;
	  %if &snpat1 ne 0 %then %do;
      %table_FillCell(&table, &rvar, PRI_TX, %bquote(&mean1 (&std1)) );
	  %end;
	  %if &snpat2 eq 0 %then %do;
	  %table_FillCell(&table, &rvar, PRI_REF, %bquote(- (- %))  );
	  %end;
	  %if &snpat2 ne 0 %then %do;
      %table_FillCell(&table, &rvar, PRI_REF, %bquote(&mean2 (&std2)) );
	  %end;
	  %if &snpat1 ne 0  and &snpat2 ne 0 %then %do;
      %table_FillCellNum(&table, &rvar, AD, &adiff, num_decimals = 1);
		  	%if &nopoolstd = 0 %then %do;
	     	   %table_FillCellNum(&table, &rvar, SD, &sdiff, num_decimals = 1);
			%end;
			%if &nopoolstd = 1 %then %do;
	     	   %table_FillCell(&table, &rvar, SD, %bquote(-));
			%end;
	  %end;
	  %else %do;
      %table_FillCell(&table, &rvar, AD, %bquote(-));
      %table_FillCell(&table, &rvar, SD, %bquote(-));
	  %end;
    %end;
  %mend;
  %tablemean(TABLE = &table, VARROW = &contvars);

  * Initialize the dist macro variables to - in case something has gone wrong;
  * If the distance dataset ;
  * then the distance will show up as - in the output tables;
  %let dist = -;

  * Use the candisc procedure to;
  * get Mahalanobis distance for the Predefined Covariates;

  ods listing close;

  %let dsid=%sysfunc(open(&dsin,in));
  %let nobs=%sysfunc(attrn(&dsid,nobs));
  %if &dsid > 0 %then %do;
		%let rc=%sysfunc(close(&dsid));
		  proc sql;
		  select count(distinct PRI_TX)
		  into :numexp
		  from &dsin;
		  quit;
		  %put &numexp;
   %end;

  %if &nobs > 0 and &numexp>1 %then %do;

 /** CHANGE DEMOGRAPHIC DATA FROM TEXT TO NUMERIC 
  	 GET NUMBER OF PREDEFINED COVARIATE CONDITIONS AND COVARIATE RX **/

  PROC SQL;

	  create table age as
	  select distinct age_cat as age
	  from &dsin
	  WHERE age is not missing;

	  create table SEX as 
	  select distinct sex
	  from &dsin
	  where sex is not missing;

  QUIT;

  %macro number(var = );
  DATA &var;
  set &var;
  num&var = _n_;
  keep &var num&var;
  RUN;

  PROC MEANS DATA = &var noprint;
  VAR num&var;
  OUTPUT OUT = temp N = N;
  RUN;
  %global num&var;
  DATA TEMP;
  SET TEMP;
  CALL SYMPUT("num&var", N);
  RUN;

  %mend;
  %number(var = age); 
  %number(var = sex);
  %put &numage &numsex;

  PROC SQL;
  CREATE TABLE numericdemo as
  SELECT D.*, numage, numsex 
  FROM &dsin as D
  INNER JOIN AGE AS A
  ON D.AGE_CAT = A.AGE
  INNER JOIN SEX AS S
  ON D.SEX = S.SEX;
  RUN;

  DATA DEMO;
  SET numericdemo;
  ARRAY A(&numage) AGE1-AGE%eval(&numage);
  ARRAY S(&numsex) SEX1-SEX%eval(&numsex);

  DO i = 1 to &numage;
  A(i) = 0;
  IF numage = i THEN A(i) = 1;
  END;

  DO i = 1 to &numsex;
  S(i) = 0;
  IF numsex = i THEN S(i) = 1;
  END;

  RUN;

    proc candisc data = demo out = mhal distance;
      class PRI_TX;
      var 	AGE1-AGE%eval(&totagecat) SEX1-SEX%eval(&numsex)
			&covar &covmed &covproc &util COMORBIDSCORE;
      ods output Dist = distance;
    run;

	/* In case of running on systems where the output dataset "distance" 
		has variable names that do not have leading "_" */
	proc transpose data = distance (drop = FromPRI_TX) out = t; run;

    proc sql noprint ;
      select col1 format 8.3 into :dist from t where monotonic() = 2;
    quit;
	%put &dist;
  
    proc datasets nolist;
      delete distance mhal t;
    run;

  %end;
  %table_FillCell(&table, MAHAL, PRI_TX, %bquote(&dist));
  ods listing;
  *** End of Mahalanobis distance code;

 %end; * if there were >0 obs in the subgroup;

%mend;


%filltables(table = Unmatched, dsname = datain2,  
            dichvars = &tabledvars, 
            contvars = &tablecvars, percentDec = 1);

%filltables(table = PRED PS Matched Caliper = .01, dsname = datain2, 
            subgroup =  MATCHEDPRED_CAL01, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = PRED PS Matched Caliper = .025, dsname = datain2,
            subgroup = MATCHEDPRED_CAL025, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = PRED PS Matched Caliper = .05, dsname = datain2, 
            subgroup = MATCHEDPRED_CAL05, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HDPS Matched Caliper = .01, dsname = datain2, 
            subgroup = MATCHEDHDPS_CAL01, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HDPS Matched Caliper = .025, dsname = datain2,
            subgroup = MATCHEDHDPS_CAL025, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HDPS Matched Caliper = .05, dsname = datain2, 
            subgroup = MATCHEDHDPS_CAL05, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HD Only Matched Caliper = .01, dsname = datain2, 
            subgroup = MATCHEDHDOnly_CAL01, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HD Only Matched Caliper = .025, dsname = datain2,
            subgroup = MATCHEDHDOnly_CAL025, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);

%filltables(table = HD Only Matched Caliper = .05, dsname = datain2, 
            subgroup = MATCHEDHDOnly_CAL05, dichvars = &tabledvars,
            contvars = &tablecvars, percentDec = 1);


* Get statistics;
%if %sysfunc(exist(dplocal.&DPID.&SITEID._&REQUESTID._estimates_&look.)) %then %do;
data stats; set dplocal.&DPID.&SITEID._&REQUESTID._estimates_&look.;
  where c_stat ne .;
run;

%let cstat_pre = PS Model did not converge;
%let cstat_hdps = PS Model did not converge;
%let cstat_hdonly = PS Model did not converge;

proc sql noprint;
  select c_stat format 8.3 into :cstat_pre from stats where pstype = "PREDEFINED" and score_model_converged = 1;
  select c_stat format 8.3 into :cstat_hdps from stats where pstype = "HDPS and PREDEFINED" and score_model_converged = 1;
  select c_stat format 8.3 into :cstat_hdonly from stats where pstype = "HDPS ONLY" and score_model_converged = 1;
quit;
%put &cstat_pre &cstat_hdps &cstat_hdonly;

PROC SQL;
		select score_model_converged into :c1-:c3
		from  dplocal.&DPID.&SITEID._&REQUESTID._estimates_&look.;
QUIT;
%put &c1 &c2 &c3;

ods graphics on;
ods path(prepend) work.templat(update);

	proc template;
	  define statgraph dualhist;
	  begingraph;
	  layout overlay / xaxisopts=(label="PS" linearopts = (viewmin=0 viewmax=1));

		/** first plot: of pri_tx (Grp1) **/
	    densityplot PrimaryTreatment / 	normal() 
										name='n1'  
	                          			legendlabel="Normal Plot of &grp1" 
	                          			lineattrs=(pattern=dash);
		histogram PrimaryTreatment/ 	name = "Histogram of &Grp1." 
										datatransparency = .8 
										binstart = 0
										binwidth = .025
										fillattrs=(color=lightblue);
	    /** first plot: of pri_ref (Grp2) **/
	    densityplot PrimaryReferent / 	normal() 
										name='n2'  
	                          		  	legendlabel="Normal Plot of &grp2";
		histogram PrimaryReferent/ 		name = "Histogram of &Grp2." 
										datatransparency = .8 
										binstart = 0
										binwidth = .025
										fillattrs=(color=lightred);
	    /** optional: add legend by specifying names **/
	    discretelegend "&grp1" "&grp2" 'n1' 'n2' "Histogram of &Grp1." "Histogram of &Grp2.";
	    endlayout;
	    endgraph;
	  end;
	run;

ods listing close;
ods rtf file = "&rtff.";
goptions reset=goptions device=png300 target=png300;

%macro createplots(tds = , setds = , pstype = );

	data &tds; set &setds;
	  if pri_tx = 1 then PrimaryTreatment = &pstype;
	  if pri_tx = 0 then PrimaryReferent = &pstype;
	run;

	proc sgrender data=&tds template=dualhist;
	run;

%mend;

title 'Histogram of Predefined PS, Unmatched Cohort';
title2 "C-Stat for Predefined: &cstat_pre";
%createplots(tds = unm_plot, setds = datain2, pstype = predefinedps);

%if &c1 = 1 %then %do;
	title 'Histogram of Predefined PS among Predefined PS Matched Cohort, Matched Cal = .01';
	title2 "C-Stat for Predefined: &cstat_pre";
	%createplots(tds = C01_plot, setds = MATCHEDPRED_CAL01, pstype = predefinedps);

	title 'Histogram of Predefined PS among Predefined PS Matched Cohort, Matched Cal = .025';
	title2 "C-Stat for Predefined: &cstat_pre";
	%createplots(tds = C025_plot, setds = MATCHEDPRED_CAL025, pstype = predefinedps);

	title 'Histogram of Predefined PS among Predefined PS Matched Cohort, Matched Cal = .05';
	title2 "C-Stat for Predefined: &cstat_pre";
	%createplots(tds = C05_plot, setds = MATCHEDPRED_CAL05, pstype = predefinedps);
%end;

title 'Histogram of HDPS+ Predefined PS, Unmatched Cohort';
title2 "C-Stat for HDPS + Predefined: &cstat_hdps";
%createplots(tds = unm_plot, setds = datain2, pstype = hdpspredefined);


%if &c2 = 1 %then %do;
	title 'Histogram of Predefined PS among HDPS + Predefined PS Matched Cohort, Matched Cal = .01';
	title2 "C-Stat for HDPS + Predefined: &cstat_hdps";
	%createplots(tds = C01_plot, setds = MATCHEDHDPS_CAL01, pstype = hdpspredefined);

	title 'Histogram of Predefined PS among HDPS + Predefined PS Matched Cohort, Matched Cal = .025';
	title2 "C-Stat for HDPS + Predefined: &cstat_hdps";
	%createplots(tds = C025_plot, setds = MATCHEDHDPS_CAL025, pstype = hdpspredefined);

	title 'Histogram of Predefined PS among HDPS + Predefined PS Matched Cohort, Matched Cal = .05';
	title2 "C-Stat for HDPS + Predefined: &cstat_hdps";
	%createplots(tds = C05_plot, setds = MATCHEDHDPS_CAL05, pstype = hdpspredefined);
%end;

title 'Histogram of hdPS Only, Unmatched Cohort';
title2 "C-Stat for HDPS Only: &cstat_hdonly";
%createplots(tds = unm_plot, setds = datain2, pstype = hdpsonly);

%if &c3 = 1 %then %do;
	title 'Histogram of Predefined PS among HDPS Only Matched Cohort, Matched Cal = .01';
	title2 "C-Stat for HDPS Only: &cstat_hdonly";
	%createplots(tds = C01_plot, setds = MATCHEDHDOnly_CAL01, pstype = hdpsonly);

	title 'Histogram of Predefined PS among HDPS Only Matched Cohort, Matched Cal = .025';
	title2 "C-Stat for HDPS Only: &cstat_hdonly";
	%createplots(tds = C025_plot, setds = MATCHEDHDOnly_CAL025, pstype = hdpsonly);

	title 'Histogram of Predefined PS among HDPS Only Matched Cohort, Matched Cal = .05';
	title2 "C-Stat for HDPS Only: &cstat_hdonly";
	%createplots(tds = C05_plot, setds = MATCHEDHDOnly_CAL05, pstype = hdpsonly);
%end;

ods rtf close;
ods listing;

%end;

%mend;

%resultsout(data_input = dplocal.&input ,
            tabledvars = &dvarlst. ,
            tablecvars = &cvarlst.);

%table_CreateTablesFromWorkbook(%NRBQUOTE(&aptab), %NRBQUOTE(&taboc));

/**************************************
 Get Total Run Time
***************************************/

    data _NULL_;
        temp=DATETIME();
        seconds=temp-&start.;
        hours=int(seconds/3600);
        minutes=int((seconds-hours*3600)/60);
        seconds2=int((seconds-hours*3600-minutes*60));
        call symput('STOP',temp);
        call symput('hours',put(hours,4.0));
        call symput('minutes',put(minutes,2.0));
        call symput('seconds',put(seconds2,2.0));
        run;

        %global tabletime;
        %let tabletime =  &hours. h &minutes. m &seconds. s;
      	%put &tabletime;

	    DATA TEMP;
		length TABLETIME $150.;
		DO i = 1 to 1;
		MONITORINGPERIOD = put(&LOOK, 2.);
		TABLETIME = "&TABLETIME";
		OUTPUT;
		DROP i;
		END;
		RUN;

		DATA MSOC.&DPID.&SITEID._&REQUESTID._TIMING;
		RETAIN MONITORINGPERIOD DATES COHORTRUNTIME ADJUSTMENTRUNTIME HDPSRUNTIME MATCHRUNTIME;
		MERGE MSOC.&DPID.&SITEID._&REQUESTID._TIMING TEMP;
		BY MONITORINGPERIOD;
		RUN;

   /** CLEAN UP **/

    proc datasets library=work nolist kill; 
    quit;
    run;

    proc datasets library = msoc;
    delete  excel_table_def table_cells 
        table_col_def table_def table_footnotes
        table_row_def;
    run;

proc printto log=log print=print;run;

%mend;

