****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: 07_sequential_analysis.sas  
*
* Created (mm/dd/yy): 11/01/2013
* Last modified: 11/01/2013
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Mini-Sentinel : Propensity Score Matching PROMPT (Prospective Routine Observational Monitoring Program Tool) 
*
*	 The Initial Developer of the Original Code is the 							  
*	 Division of Pharmacoepidemiology and Pharmacoeconomics, 					  
*	 Department of Medicine, Brigham and Women's Hospital and 					  
*	 Harvard Medical School					  									  
*																				  
*	 Module Leads: Joshua J Gagne, Shirley V Wang, Sebastian Schneeweiss			  
*	 Programmer:   Shirley V Wang												  
*	 Uses Pharmacoepidemiology Toolbox macros, created by Jeremy Rassen	
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;

/********************************************************************************
PROSPECTIVE MONITORING ANALYSIS QUERY CALL

REQUIRED INPUTS:

	1. Define MSOC-assigned request ID
	2. Specify number of contributing Data Partners and edit DPID and SITEID	
	3. Specify analysis type
	4. Assign paths to reflect the relevant libraries and folders
 	5. Specify which monitoring periods to analyze		
	
 ********************************************************************************/

*System Options;
options mprint 	linesize=190 pagesize=50 compress=yes reuse=no symbolgen ERRORS=25
				sastrace=',,,d' sastraceloc=saslog NOSTSUFFIX
				FORMCHAR="|----|+|---+=|-/\<>*";

*-----------------------------------
* Assign STANDARD Parameters
*-----------------------------------;

/* 1. Define MSOC-assigned request ID */
%let REQUESTID = [enter-requestid];


/* 2. Specify number of contributing Data Partners and edit DPID and SITEID */	                                                 
%let NumAgg = ;

%let DPID1 = [enter-dpid];	
%let SITEID1 = [enter-siteid];

%let DPID2 = [enter-dpid];	
%let SITEID2 = [enter-siteid];

%let DPID3 = [enter-dpid];	
%let SITEID3 = [enter-siteid];

 /*-------------------------------------------------------------------*\
 ||                                                                   ||
 ||  DATA PARTNERS                                 DPID      SITEID   ||
 ||  -----------------------------------------------------------------||
 ||   Healthcore (one site)                         HC         OS     ||
 ||   Humana (one site)                             HU         OS     ||
 ||   Aetna (one site)                              AE         OS     ||
 ||   Optum (one site)                              OP         OS     ||
 ||   Vanderbilt University (one site)              VB         OS     ||
 ||   HMORN                                                           ||
 ||      Group Health Cooperative                   HM         GHC    ||
 ||      Fallon Community Health Plan               HM         MPCI   ||   
 ||      Henry Ford Health System                   HM         HFHS   || 
 ||      Lovelace Health System                     HM         LCF    || 
 ||      Marshfield Clinic                          HM         MCRF   ||
 ||      HealthPartners                             HM         HPRF   ||
 ||      Harvard Pilgrim Health Care                HM         HPHC   ||
 ||   Kaiser Permanente                                               ||         
 ||      Kaiser DCC                                 KP         DCC    ||
 ||      Kaiser Permanente Colorado                 KP         CO     ||
 ||      Kaiser Permanente Georgia                  KP         GA     ||
 ||      Kaiser Permanente Hawaii                   KP         HI     ||
 ||      Kaiser Permanente Northern California      KP         NC     ||
 ||      Kaiser Permanente Northwest                KP         NW     ||
 ||      Kaiser Permanente Mid Atlantic             KP         MA     ||
 \*-------------------------------------------------------------------*/
 
 
/*
3. Specify analysis type 													
	  	Leave blank if this is an AS TREATED analysis									
 	  	Enter 1 if this is an INTENTION TO TREAT (ITT) analysis
*/				
%let itt = ;				

	
/* 4. Assign paths to reflect the relevant libraries and folders */
/* IMPORTANT NOTE: END OF PATH SEPARATORS ARE REQUIRED!! */
/*		
			Windows-based platforms:  
				"\", e.g. "C:\user\sas\" and not "C:\user\sas";					
			Unix-based platforms:           										
				"/", e.g."/home/user/sas/" and not "/home/user/sas";	
*/

/* Path locations for each Data Partner's MSOC subdirectory */

/*	For Data Partner 1 */																
%let msoc1 = /your-path/dpid-siteid/msoc/;
libname msoc1 "&MSOC1.";

/*	For Data Partner 2 */																
%let msoc2 = /your-path/dpid-siteid/msoc/;
libname msoc2 "&MSOC2.";

/*	For Data Partner 3 */																
%let msoc3 = /your-path/dpid-siteid/msoc/;
libname msoc3 "&MSOC3.";

/* etc. */

/* Directory to which files will be written */
%let output = /your-path/msoc/;
libname output "&output.";

/* Directory containing suite of Cohort Matching Modular Programs */
%let query = /your-path/sasprograms/;

/* SAS %include statement for sequential monitoring period for this REQUESTID */
%include "&query.01_monitoring.sas";

/* Directory containing Pharmacoepidemiology Toolbox Macros */
/* IMPORTANT NOTE: PATHNAME CANNOT HAVE SPACES!! */
%let toolbox = /your-path/hdps/toolbox/; 

/* SAS %include statements; no editing needed */
%include "&toolbox.matching_v214.sas";
%include "&toolbox.java_utils_v214.sas";
%let jar = &toolbox.pharmacoepi_v214.jar;
%javautils_InitClasspathUpdate;
%javautils_AddToClasspath(&jar);
%toolbox_Start(%QUOTE(&jar));

/* Directory for temporary hd-PS files */ 			
%let temphdps = /your-path/hdps/temphdps/; 
libname temphdps "&tempHDPS";


/*******************************************************************************
********************************************************************************
ANALYSIS PROGRAM:
PURPOSE: Obtain Sequential Estimates By Data Partner and Monitoring Period: 

Output results for: 	
	1. Unadjusted incidence rate differences stratified by DP 
	2. Adjusted incidence rate differences stratified by DP and 1:1 or 1:n variable matchid 
	3. Adjusted incidence rate differences stratified by DP and deciles of PS 
			(Report decile specific estimates and summary estimate) 
	4. Unadjusted Cox model stratified by DP 
	5. Adjusted Cox models stratified by DP and 1:1 or 1:n variable matchid 
	6. Adjusted Cox model stratified by DP and deciles of PS 
			(Report decile specific estimates and summary estimate) 
	7. NNT, ARR, PARR 

	For each of items 1-7, provide analysis results for:
	1. full cohort 
	2. by pre-specified subgroups

********************************************************************************
********************************************************************************/
%macro sequential(first = 1, period = 1, subgroup = , PS = hdpspredefined, caliper = 2); 

/* Was this requestid package run for an ASTREATED or ITT analysis? */

	%if &ITT = %str() %then %let a_type = ASTREATED;
	%if &ITT = 1 %then %let a_type = ITT;

/* Which PS and matching caliper was requested? If none specified, set to default */
	
	%if &PS = %str() %then %let PS = hdpspredefined;
	%if &caliper = %str() %then %let caliper = 2;

/* Make sure base data sets for appending results are clear before starting*/

	proc datasets library = work; 
		 delete estimates; 
	run;

/* Append the output data returned to the MSOC from the Data Partners 
   for each sequential monitoring period where analysis results are requested */

%do i = &first %to &period;

	* Bring in monitoring period dates for the results tables;
		%let look = &i; 
		%monitoringperiod();

	* Create a combined dataset with new id based on dpid and siteid for stratification;
		proc datasets library = work; delete agg_dp_&i; run;

		%macro newdata();

			%do dp = 1 %to &numagg; 

				%let dpid = &&dpid&dp;
				%let siteid = &&siteid&dp;
				%put &dpid &siteid;

				%if %sysfunc(exist(msoc&dp..&DPID.&SITEID._&REQUESTID._matched_&i)) %then %do; 

					data dp_&dp; 
					set msoc&dp..&DPID.&SITEID._&REQUESTID._matched_&i;
					length dpidsiteid $32.;
					dpidsiteid = "&DPID."||"&SITEID.";
					label dpidsiteid = "unique identifier for Data Partner Site";
					label age_cat = "Age Categories";
					run; 

					PROC APPEND DATA = dp_&dp base = agg_dp_&i force; RUN;

				%end; * if data exists;

			%end;* dp to &numagg;

	* Get deciles;
			PROC SORT DATA = agg_dp_&i; by dpidsiteid; run;

			PROC RANK DATA = agg_dp_&i out = d group = 10;
			by dpidsiteid;
			var &ps;
			ranks deciles;
			RUN;
			
			data agg_dp_&i;
			set d;
			deciles = deciles+1;
			run; 

	* Put studyclass names into macro variables;
			%global e0 e1;
			proc sql;
				select distinct studyclass, exposure
				into :e0-:e1, :b0-:b1
				from agg_dp_&i
				order by exposure;
			quit;
			%put &e0 &e1 &b0 &b1;

		%mend newdata; 

		%newdata(); 

/***********************************************************************
	Get Estimates for Incidence Rate Differences and HR 
************************************************************************/

	%macro stratified_analysis(strata = );

	* Default no additional subgroups analysis;
		%let group = ;

	* Identify number of strata for the analysis;
		%let wordcount=%sysfunc(countw(&strata));
		%put &wordcount;

		%global nomissing all_s;
		%let nomissing = ;
		%let all_s = ;
		%let s = %scan(&strata, 1, ', ');
		%let nomissing = &s is not missing;
		%let all_s = &s;
			%do w = 2 %to &wordcount;
				%let s =%scan(&strata,&w,', ');
				%let nomissing = &nomissing and &s is not missing;
				%let all_s = &all_s &s;
			%end;

		%put &nomissing;
		%put &all_s;

		PROC SQL;

			create table strataby as
			select distinct &strata
			from agg_dp_&i
			where &nomissing
			order by &strata;

			select distinct count(*) into :nums
			from strataby;

		QUIT; 

		%let numstrata = %eval(&nums);
		%put &numstrata;

	* if there is at least one strata then proceed;

		%if &numstrata >0 %then %do;

		* Identify events and person time by exposure and data partner;
			%do e = 0 %to 1; 

				proc sql;
					create table n_case_&e as
					select &strata, sum(event_&a_type.) as n_case_&e, count(*) as n_&e
					from agg_dp_&i
					where exposure = &e. and &nomissing
					group by &strata.
					order by &strata.;
					
					create table fu_&e as
					select &strata, sum(followuptime_&a_type.) as fu_&e
					from agg_dp_&i
					where exposure = &e. and &nomissing
					group by &strata.
					order by &strata.;

				quit;

			%end; 

		* 1. Calculate weight within strata ;
			* 2. Calculate difference within strata ;
				* 3. Calculated weighted difference;
			
			data stratatotals;
				merge n_case_1 n_case_0 fu_1 fu_0;
				by %sysfunc(compress(&strata., ","));
				N1sq = fu_1**2;
				N0sq = fu_0**2;
				numerator = (N1sq*N0sq);
				denominator =  (n_case_1*N0Sq + n_case_0*N1sq) ;
				weight = 0;
				if denominator > 0 then do;
	                weight = numerator/denominator ; *1.;
				    diff = (n_case_1/fu_1) - (n_case_0/fu_0); *2.;
				    weighted_diff = weight*diff; *3.;
				end;
			run; 

		* get number of patients, cases, and follow up time by exposure;
			proc sql;
				select sum(n_1) into: N_&e1.
				from stratatotals;
				select sum(n_0) into: N_&e0.
				from stratatotals;
			quit;

			proc means data = stratatotals noprint;
			var n_case_1 fu_1 n_case_0 fu_0;
			output out = sum sum = n_case_&e1 fu_&e1 n_case_&e0 fu_&e0;
			run;

		* Obtain stratified incidence rate difference estimate with confidence intervals;
			* include counts of patients, cases and follow up time in output dataset;

			proc sql;
				select sum(weight) into :denom
				from stratatotals;
				select sum(weighted_diff) into :num
				from stratatotals;
			quit;

			%let stratifiedratediff = %SYSEVALF(&num./&denom.);
			%put &stratifiedratediff;
			%let var = %SYSEVALF(1/&denom.);
			%put &var;
			%let lower = %SYSEVALF(&stratifiedratediff. - 1.96*(&var.**.5));
			%let upper = %SYSEVALF(&stratifiedratediff. + 1.96*(&var.**.5));
			%put &lower &upper;

			data rd_est;
				retain 	MonitoringPeriod Analysis RD_95CI 
						n_&e1 n_case_&e1 fu_&e1 n_&e0 n_case_&e0 fu_&e0;
				set sum (drop = _type_ _freq_);
				length MonitoringPeriod $50.;
				length Analysis $250.;
				length RD_95CI $50.;
				MonitoringPeriod = "Period"||put(&look., 2.)||": &startfollowup. - &enddate.";
				Analysis = "Summary Estimate: stratified by &strata.";			
				stratifiedrd = put(&stratifiedratediff., 8.5); 
				lower = put(&Lower, 8.5);
				upper =put(&Upper, 8.5);
				RD_95CI = (stratifiedrd||" ("||lower||", "||upper||")");
				n_&e1 = %eval(&&n_&e1.);
				n_&e0 = %eval(&&n_&e0.);
				label MonitoringPeriod = "Sequential Monitoring Period";
				label Analysis = "Analysis Type";
				label rd_95CI = "Incidence Rate Difference (95% Confidence Interval)";
				label n_&e1 = "N new users &e1.";
				label n_&e0 = "N new users &e0.";
				label n_case_&e1 = "N Events &e1.";
				label n_case_&e0 = "N Events &e0.";
				label fu_&e1 = "Person-Time &e1.";
				label fu_&e0 = "Person-Time &e0.";

				if n_&e1. >0 then risk1 =n_case_&e1./n_&e1.;
				label risk1 = "Risk among new users of &e1.";
				if n_&e0. > 0 then risk0 = n_case_&e0./n_&e0.;
				label risk0 = "Risk among new users of &e0.";
				if n_&e0.+n_&e1. > 0 then poprisk = (n_case_&e0.+n_case_&e1.)/(n_&e0.+n_&e1.);
				label poprisk = "Pop. Risk among new users of &e1. AND &e0.";
				diff =  risk1 - risk0 ;
				label diff = "Difference in Risk: &e1. - &e0.";
				if diff > 0 then NNT = 1/diff;
				label NNT = "Number needed to treat (or harm)";
				if risk1 > 0 then AR = diff/risk1;
				label AR = "Attributable Risk %";
				if poprisk > 0 then PAR = (poprisk-risk0)/poprisk;
				label PAR = "Population Attributable Risk %";

				format 	risk1 best8.4 risk0 best8.4 poprisk best8.4 diff best8.4 
						nnt best8.4 ar percentn12.2 par percentn12.2
						n_&e1 comma8. n_&e0 comma8. 
	                   	n_case_&e1 comma8. n_case_&e0 comma8.
					   	fu_&e1 comma8. fu_&e0 comma8.;
				keep 	MonitoringPeriod Analysis RD_95CI 
						n_&e1 n_case_&e1 fu_&e1 n_&e0 n_case_&e0 fu_&e0
						risk1 risk0 poprisk diff nnt ar par;
			run;

		* Obtain stratified cox PH estimates with confidence intervals;
			ods listing close;

			proc phreg data = agg_dp_&i;
				where &nomissing;
				strata &all_s;
				model followuptime_&a_type.*event_&a_type.(0) = EXPOSURE/ ties = breslow rl;
				ods output ParameterEstimates=pest;
			run;

			ods listing;

			DATA coxPHest; 
				RETAIN MonitoringPeriod ANALYSIS HR_95CI HR_pvalue;
				SET pest (RENAME = (probchisq = HR_pvalue) 
						  RENAME = (estimate = HR_coef)
						  RENAME = (stderr = HR_se));
				FORMAT HAZARDRATIO HRLOWERCL HRUPPERCL 5.2;
				length MonitoringPeriod $50.;
				length Analysis $250.;
				length HR_95CI $50.;
				MonitoringPeriod = "Period"||put(&look., 2.)||": &startfollowup. - &enddate.";
				Analysis = "Summary Estimate: stratified by &strata.";
				HR_95CI = put(HAZARDRATIO, 5.2)|| " ("||put(HRLOWERCL, 5.2)||", "|| put(HRUPPERCL, 5.2)||")";
				label MonitoringPeriod = "Sequential Monitoring Period";
				label Analysis = "Analysis Type";
				label hr_95CI = "Hazard Ratio (95% Confidence Interval)";
				label HR_coef = "Coefficient from PHREG";
				label HR_pvalue = "WALD p-value";
				label HR_se = "StdErr of Coefficient";
				KEEP MonitoringPeriod ANALYSIS HR_95CI HR_pvalue HR_coef HR_se;
			RUN;

			/* Bring together RD, HR and pop health estimates from different analyses */
			PROC SQL;
				create table temp as
				select r.*, HR_95CI, HR_pvalue, HR_coef, HR_se
				from rd_est as r
				inner join coxPHest as c
				on r.monitoringperiod = c.monitoringperiod
	                and r.analysis = c.analysis;
			QUIT;

			proc append data = temp base = estimates force; run;
			
			/* add decile and prespecified subgroup specific estimates if relevant 
				always add separated analyses by DP if more than one DP contributed data to analyses*/
			%if %index(&strata., deciles)>0 
				or %index(&strata., &pregroup.)>0 
					or (&strata = %quote(dpidsiteid) and &numAGG >1)
						or (&strata = %quote(dpidsiteid, &ps.matchid1_&caliper.) and &numAgg>1) 
							or (&strata = %quote(dpidsiteid, &ps.matchid100_&caliper.) and &numAgg >1)
								%then %do;

					%if %index(&strata., deciles)>0 %then %let group = deciles;
					%if %index(&strata., &pregroup.)>0 %then %let group = &pregroup.;
					%if &strata = %quote(dpidsiteid) 
							or &strata = %quote(dpidsiteid, &ps.matchid1_&caliper.) 
								or &strata = %quote(dpidsiteid, &ps.matchid100_&caliper.) %then %let group = dpidsiteid;

				proc sort data = stratatotals; by &group.; run;

		** get number of patients, cases, and follow up time by exposure and decile/subgroup;		

				PROC SQL;
					create table total as
					select &group, sum(n_1) as n_&e1, sum(n_0) as n_&e0
					from stratatotals
					group by &group;
				QUIT;

				proc means data = stratatotals noprint;
					by &group;
					var n_case_1 fu_1 n_case_0 fu_0;
					output out = sum sum = n_case_&e1 fu_&e1 n_case_&e0 fu_&e0;
				run;

		* Obtain stratified incidence rate difference estimate with confidence intervals;
			* include counts of patients, cases and follow up time in output dataset;

				proc sql;
					create table num_denom as
					select &group, sum(weight) as denom, sum(weighted_diff) as num
					from stratatotals
					group by &group;
				quit;

				data rd; 
					set num_denom;
					stratifiedrd = 0;
					var = 0;
					lower = 0;
					upper = 0;
					if denom > 0 then do;
						stratifiedratediff = num/denom; 
						var = 1/denom;
		 				l = stratifiedratediff - 1.96*(var**.5);
						u = stratifiedratediff + 1.96*(var**.5);
					end;
				run; 

				data rd_est;
					retain 	MonitoringPeriod Analysis RD_95CI 
							n_case_&e1 fu_&e1 n_case_&e0 fu_&e0
							stratifiedrd lower upper;
					merge sum (drop = _type_ _freq_)
						  rd
						  total;
					by &group;
						length MonitoringPeriod $50.;
						length Analysis $250.;
						length RD_95CI $50.;
						MonitoringPeriod = "Period"||put(&look., 2.)||": &startfollowup. - &enddate.";
						Analysis = "Stratified by &strata.: Estimate among &group. ="||&group.;
						RD_95CI = (put(stratifiedratediff, 8.5)||" ("||put(l, 8.5)||", "||put(u, 8.5)||")");
						label MonitoringPeriod = "Sequential Monitoring Period";
						label Analysis = "Analysis Type";
						label rd_95CI = "Incidence Rate Difference (95% Confidence Interval)";
						label n_&e1 = "N new users &e1.";
						label n_&e0 = "N new users &e0.";
						label n_case_&e1 = "N Events &e1.";
						label n_case_&e0 = "N Events &e0.";
						label fu_&e1 = "Person-Time &e1.";
						label fu_&e0 = "Person-Time &e0.";

						if n_&e1. >0 then risk1 =n_case_&e1./n_&e1.;
						label risk1 = "Risk among new users of &e1.";
						if n_&e0. > 0 then risk0 = n_case_&e0./n_&e0.;
						label risk0 = "Risk among new users of &e0.";
						if n_&e0.+n_&e1. > 0 then poprisk = (n_case_&e0.+n_case_&e1.)/(n_&e0.+n_&e1.);
						label poprisk = "Pop. Risk among new users of &e1. AND &e0.";
						diff =  risk1 - risk0 ;
						label diff = "Difference in Risk: &e1. - &e0.";
						if diff > 0 then NNT = 1/diff;
						label NNT = "Number needed to treat (or harm)";
						if risk1 > 0 then AR = diff/risk1;
						label AR = "Attributable Risk %";
						if poprisk > 0 then PAR = (poprisk-risk0)/poprisk;
						label PAR = "Population Attributable Risk %";

					format 	risk1 best8.4 risk0 best8.4 poprisk best8.4 diff best8.4 
							nnt best8.4 ar percentn12.2 par percentn12.2
							n_&e1 comma8. n_&e0 comma8. 
		                   	n_case_&e1 comma8. n_case_&e0 comma8.
						   	fu_&e1 comma8. fu_&e0 comma8.;
					keep 	MonitoringPeriod Analysis RD_95CI 
							n_&e1 n_case_&e1 fu_&e1 n_&e0 n_case_&e0 fu_&e0
		                    risk1 risk0 poprisk diff nnt ar par;
				run;

				proc sort data = agg_dp_&i; by &group; run;

				ods listing close;

				proc phreg data = agg_dp_&i;
					by &group;
					where &nomissing;
					strata &all_s;
					model followuptime_&a_type.*event_&a_type.(0) = EXPOSURE/ ties = breslow rl;
					ods output ParameterEstimates=pest;
				run;

				ods listing;

				DATA coxPHest; 
					RETAIN MonitoringPeriod ANALYSIS HR_95CI HR_pvalue;
					SET pest (RENAME = (probchisq = HR_pvalue) 
						  	  RENAME = (estimate = HR_coef)
						  	  RENAME = (stderr = HR_se));
					FORMAT HAZARDRATIO HRLOWERCL HRUPPERCL 5.2;
					length MonitoringPeriod $50.;
					length Analysis $250.;
					length HR_95CI $50.;
					MonitoringPeriod = "Period"||put(&look., 2.)||": &startfollowup. - &enddate.";
					Analysis = "Stratified by &strata.: Estimate among &group. ="||&group.;
					HR_95CI = put(HAZARDRATIO, 5.2)|| " ("||put(HRLOWERCL, 5.2)||", "|| put(HRUPPERCL, 5.2)||")";
					label MonitoringPeriod = "Sequential Monitoring Period";
					label Analysis = "Analysis Type";
					label hr_95CI = "Hazard Ratio (95% Confidence Interval)";
					label HR_coef = "Coefficient from PHREG";
					label HR_pvalue = "WALD p-value";
					label HR_se = "StdErr of Coefficient";
					KEEP MonitoringPeriod ANALYSIS HR_95CI HR_pvalue HR_coef HR_se;
				RUN;

			/* Bring together rows from different analyses by decile/subgroup */
				PROC SQL;
					create table temp as
					select r.*, HR_95CI, HR_pvalue, HR_coef, HR_se
					from rd_est as r
					inner join coxPHest as c
					on r.monitoringperiod = c.monitoringperiod
					and r.analysis = c.analysis;
				QUIT;

				proc append data = temp base = estimates force; run;

		%end;
		
		%end; * if at least one strata exists;

		%if &numstrata<=0 %then %do;

			data temp;
				length MonitoringPeriod $50.;
				length Analysis $250.;
				length RD_95CI $50.;
				length HR_95CI $50.;
				MonitoringPeriod = "Period"||put(&look., 2.)||": &startfollowup. - &enddate.";
				label MonitoringPeriod = "Sequential Monitoring Period";
				Analysis = "Summary Estimate: stratified by &strata.";
				label Analysis = "Analysis Type";	
				RD_95CI = "";
				label rd_95CI = "Incidence Rate Difference (95% Confidence Interval)";
				HR_95CI = "";
				label hr_95CI = "Hazard Ratio (95% Confidence Interval)";
				HR_pvalue = .;
				label HR_pvalue = "WALD p-value";
				HR_coef = .;
				label HR_coef = "Coefficient from PHREG";
				HR_se = .;
				label HR_se = "StdErr of Coefficient";
				n_&e1 = .;
				label n_&e1 = "N new users &e1.";
				n_case_&e1 = .;
				label n_case_&e1 = "N Events &e1.";
				fu_&e1 = .;
				label fu_&e1 = "Person-Time &e1.";
				n_&e0 = .;
				label n_&e0 = "N new users &e0.";
				n_case_&e0 = .;
				label n_case_&e0 = "N Events &e0.";
				fu_&e0 = .;
				label fu_&e0 = "Person-Time &e0.";
				risk1 = .;
				label risk1 = "Risk among new users of &e1.";
				risk0 = .;
				label risk0 = "Risk among new users of &e0.";
				poprisk = .;
				diff = .;
				label diff = "Difference in Risk: &e1. - &e0.";
				nnt = .;
				label NNT = "Number needed to treat (or harm)";
				ar = .;
				label AR = "Attributable Risk %";
				par = .;
				label PAR = "Population Attributable Risk %";
			run;
	
			proc append data = temp base = estimates force; run;

		%end; * if 0 patients meet this strata criterion;

		%mend stratified_analysis;

		%global pregroup; 
		%let pregroup = nogroup;

		/** Unadjusted - stratified by DP **/ 
		%stratified_analysis(strata = %quote(dpidsiteid));

		/**	Adjusted - stratified by DP and 1:1 or 1:n variable matchid **/
		%stratified_analysis(strata = %quote(dpidsiteid, &ps.matchid1_&caliper.));
		%stratified_analysis(strata = %quote(dpidsiteid, &ps.matchid100_&caliper.));

		/** Adjusted - stratified by DP and deciles of PS 			  
			(Report decile specific estimates and summary estimate) 					  **/
		%stratified_analysis(strata = %quote(dpidsiteid, deciles));

		/** Additional subgroup? **/

		%if &subgroup ne %str() %then %do;

			* Identify number of subgroup analyses desired;
			%let numsub=%sysfunc(countw(&subgroup));
			%put &numsub;

			%do whichsub = 1 %to &numsub;
				%let pregroup =%scan(&subgroup.,&whichsub.,', '); 

				/* Re-match within subgroup, dpid, and time */
				data temp;
					set agg_dp_&i.;
					matchwithin = compress(&pregroup.|| dpidsiteid || time, " -");
					patid = left(put(_N_, 8.));
					drop &ps.matchid1_&caliper. &ps.matchid100_&caliper.;
				run;

				proc sql;
					select distinct matchwithin 
					into :matchwithin separated by ' '
					from temp;
				quit;
				
				%let nummatchstrata = %sysfunc(countw(&matchwithin, ' '));
				%put &nummatchstrata;
				
				%macro wrapper();
					%do whichstrata = 1 %to &nummatchstrata;

						%let thisstrata =%scan(&matchwithin.,&whichstrata.,' '); 

						data tempmatch;
						set temp;
						where matchwithin = "&thisstrata.";
						run;

						%IF &caliper = 1 %THEN %LET cal = .01;
						%IF &caliper = 2 %THEN %LET cal = .025;
						%IF &caliper = 3 %THEN %LET cal = .05;

							%do variable = 0 %to 1;

								%if &variable = 0 %then %let v = 1;
								%if &variable = 1 %then %let v = 100;

								%match_NearestNeighborMatch(
								in_dataset         = tempmatch,
								out_dataset        = matchcaliper_&v.,
								working_directory  = %QUOTE(&temphdps),
								var_patient_id     = patid,
								var_exposure       = exposure,
								var_ps             = &PS.,
								exp_groups         = 1 0,
								caliper            = &cal.,
								ratio              = &v.,
								fixed_ratio        = 0);


									/** IF AT LEAST ONE MATCH FOUND THEN DO **/
								%if %sysfunc(exist(matchcaliper_&v.)) %then %do;
									DATA matchcaliper_&v.;
									SET matchcaliper_&v.;
									length &PS.matchid&v._&caliper. $50.;
									&PS.matchid&v._&caliper. = COMPRESS(TIME||SET_NUM);
									KEEP patid &PS.matchid&v._&caliper.;
									RUN;    
								%end;
								
								/** IF NO MATCHES FOUND THEN DO **/
								%else %do;
									DATA matchcaliper_&v.;
									SET tempmatch (KEEP =patid);
									length &PS.matchid&v._&caliper. $50.;
									&PS.matchid&v._&caliper.  = " ";
									KEEP patid &PS.matchid&v._&caliper.;
									RUN; 
								%end;

								proc append data = matchcaliper_&v. base = matchratio&v.; run;

							%end; * variable;

					%end; /* do whichstrata = 1 to &nummatchstrata */
					%mend;
					%wrapper;

					proc sort data = matchratio1; by patid; run;
					proc sort data = matchratio100; by patid; run;
					proc sort data = temp; by patid; run;

					data agg_dp_&i.;
					merge temp matchratio1 matchratio100;
					by patid;
					drop patid;
					run;

					proc datasets library = work; delete matchratio1 matchratio100; run;

				%stratified_analysis(strata = %quote(dpidsiteid, &pregroup.));
				%stratified_analysis(strata = %quote(dpidsiteid, &ps.matchid1_&caliper., &pregroup.));
				%stratified_analysis(strata = %quote(dpidsiteid, &ps.matchid100_&caliper., &pregroup.));

			%end;  /* %scan(&subgroup.,&whichsub.,', ') */

		%end; /* if subgroup ne %str() */

	* Make Table of Contents dataset to print to excel worksheet;

		proc contents data = agg_dp_&i
		    out = varname (keep = name label);
		run;
  
		data index1;
		length index $250.;
		do ix = 1 to 10;
	     if ix = 1 then index = "Table of Contents";
	     if ix = 2 then index = "1. Sequential Estimates";
		 if ix = 3 then index = "2. Decile Specific";
		 if ix = 4 then index = "3. Prespecified Subgroups";
		 if ix = 5 then index = " ";
		 if ix = 6 then index = "Variable Name Definitions";
		 if ix = 7 then index = "dpidsiteid = unique identifier for Data Partner Site";
		 if ix = 8 then index = "&ps.matchid1_&caliper. = matched pair identifier for 1:1 match on &PS score";
		 if ix = 9 then index = "&ps.matchid100_&caliper. = matched set identifier for 1:100 variable match on &PS score";
		 if ix = 10 then index = "decile = decile of &PS propensity score";
        output;
		keep index;
		label index = "00"x;
    	end;
		run;


		%if &numAgg>1 %then %let allsub = %quote(dpidsiteid, &subgroup);
		%else %let allsub = &subgroup;
		%if &allsub eq %str() %then %let totsub=0;
		%else %let totsub=%sysfunc(countw(&allsub));
		%put &totsub;
	
		%if &totsub > 0 %then %do;
			data index2;
			set varname;
			length index $250.;
			%do ns = 1 %to &totsub;
				%global sub&ns;
				%let sub&ns =%scan(&allsub.,&ns.,', '); 
				if upcase(name) = upcase("&&sub&ns") then index = "Subgroup analyses by &&sub&ns. = "||label;
			%end;
			if index ne " ";
			keep index;
			label index = "00"x;
			run;

			data index; 
			set index1 index2; 
			run;

		%end;

		%if &totsub = 0 %then %do;

			data index; 
			set index1 ; 
			run;

		%end;

	%end; /*%do i = &first %to &period*/

* Save analysis results as SAS dataset in output library folder;

	data output.estimates_&requestid.;
	set estimates;
	run; 


/**********************************************************************
	Output Results to XLS worksheets
***********************************************************************/
	%macro out();
    title; footnote;
	%let filename = &REQUESTID._&numAgg.DP_Period&first.toPeriod&period._analysis_;

    ods listing close;
	ods tagsets.excelxp file="&output.&filename.&sysdate9..xls" style=printer
	    options( embedded_titles='yes' embedded_footnotes='yes'
				 sheet_interval='none' sheet_name='Index' 
				 absolute_column_width = "250" 
				 width_fudge = "1" width_points = "1" 
				 autofit_height = 'yes' center_horizontal = 'no');

	proc print data = index noobs label; 
	var index;
	run;

	/****************************************************************
	 Sequential Monitoring Overall Estimates
	*****************************************************************/	
    ods tagsets.excelxp options(sheet_interval='none' sheet_name='1. Sequential Estimates'
								absolute_column_width = "150, 150, 130, 110, 100, 120, 130, 90, 90, 90, 90, 90, 90, 130, 130, 130, 175" 
								width_fudge = "1" width_points = "1" 
								autofit_height = 'yes' center_horizontal = 'yes');
	
	title1 "REQUESTID = &REQUESTID.";
	title2 "Contributing Data Partners:";

	%do i = 1 %to &numagg;
	title%eval(&i+2) "DPID = &&DPID&i SITEID = &&SITEID&i";
	%end;

	title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - Unadjusted analysis";
	proc print data = output.estimates_&requestid. noobs label; 
	where analysis = "Summary Estimate: stratified by dpidsiteid";
	var MonitoringPeriod ANALYSIS RD_95CI HR_95CI NNT AR PAR 
		n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
		risk1 risk0 diff poprisk;
	run;

	title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - &PS 1:1 matched analysis";
	proc print data = output.estimates_&requestid. noobs label; 
	where analysis = "Summary Estimate: stratified by dpidsiteid, &ps.matchid1_&caliper."; 
	var MonitoringPeriod ANALYSIS RD_95CI HR_95CI NNT AR PAR 
		n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
		risk1 risk0 diff poprisk;
	run;

	title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - &PS 1:100 Variable matched analysis";
	proc print data = output.estimates_&requestid. noobs label; 
	where analysis = "Summary Estimate: stratified by dpidsiteid, &ps.matchid100_&caliper."; 
	var MonitoringPeriod ANALYSIS RD_95CI HR_95CI NNT AR PAR 
		n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
		risk1 risk0 diff poprisk;
	run;

	title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - &PS Deciles analysis";
	proc print data = output.estimates_&requestid. noobs label; 
	where index(analysis, "Summary") > 0 and index(analysis, "decile") > 0;
	var MonitoringPeriod ANALYSIS RD_95CI HR_95CI NNT AR PAR 
		n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
		risk1 risk0 diff poprisk;
	run;

	title;

	/****************************************************************
	Estimates by PS Decile
	*****************************************************************/
	ods tagsets.excelxp options(sheet_interval='none' sheet_name='2. Decile Specific'
								absolute_column_width = "150, 150, 130, 110, 100, 120, 130, 90, 90, 90, 90, 90, 90, 130, 130, 130, 175" 
								width_fudge = "1" width_points = "1" 
								autofit_height = 'yes' center_horizontal = 'yes');

	title1 "REQUESTID = &REQUESTID.";
	title2 "Contributing Data Partners:";

	%do i = 1 %to &numagg;
	title%eval(&i+2) "DPID = &&DPID&i SITEID = &&SITEID&i";
	%end;

	title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - By &PS Decile";
	proc print data = output.estimates_&requestid. noobs label; 
	where index(analysis, "decile") > 0;
	var MonitoringPeriod ANALYSIS RD_95CI HR_95CI  
		n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
		;
	run;

	title;

	/****************************************************************
	Estimates by Predefined Subgroup
	*****************************************************************/
	%if &totsub>0 %then %do;

	    ods tagsets.excelxp options(sheet_interval='none' sheet_name='3. Prespecified Subgroups'
									absolute_column_width = "150, 200, 130, 110, 100, 120, 130, 90, 90, 90, 90, 90, 90, 130, 130, 130, 175" 
									width_fudge = "1" width_points = "1" 
									autofit_height = 'yes' center_horizontal = 'yes');

		title1 "REQUESTID = &REQUESTID.";
		title2 "Contributing Data Partners:";

		%do i = 1 %to &numagg;
		title%eval(&i+2) "DPID = &&DPID&i SITEID = &&SITEID&i";
		%end;

		* Identify number of subgroup analyses - add subgroups by DP;

		%do bysub = 1 %to &totsub;
			%global printsub; 
			%let printsub =%scan(&allsub., &bysub., ', '); 
			%put &printsub;

		title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - Subgroup Analysis by &printsub. - Unadjusted";

		proc print data = output.estimates_&requestid. noobs label; 
		where index(analysis, "&printsub.") >0 
			and index(analysis, "&ps.matchid1_&caliper.") = 0 
				and index(analysis, "&ps.matchid100_&caliper.") = 0
					%if &printsub = dpidsiteid %then %do;
						and count(analysis, ",")=0;
					%end;
						;
		var MonitoringPeriod ANALYSIS RD_95CI HR_95CI  
			n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
			;
		run;

		title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - Subgroup Analysis by &printsub. - &PS 1:1 matched";

		proc print data = output.estimates_&requestid. noobs label; 
		where index(analysis, "&ps.matchid1_&caliper.") > 0
			and index(analysis, "&printsub.") >0 
			%if &printsub = dpidsiteid %then %do;
				and count(analysis, ",")=1
			%end;
				;
		var MonitoringPeriod ANALYSIS RD_95CI HR_95CI  
			n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
			;
		run;

		title%eval(2+&numagg.+1) "Sequential Monitoring Period Estimates - Subgroup Analysis by &printsub. - &PS 1:100 Variable matched";

		proc print data = output.estimates_&requestid. noobs label; 
		where index(analysis, "&printsub.") >0 
			and index(analysis, "&ps.matchid100_&caliper.") > 0
			%if &printsub = dpidsiteid %then %do;
				and count(analysis, ",")=1
			%end;
				;
		var MonitoringPeriod ANALYSIS RD_95CI HR_95CI  
			n_&e1. n_case_&e1. fu_&e1. n_&e0. n_case_&e0. fu_&e0. HR_pvalue HR_coef HR_se
			;
		run;
		title;

		%end; * bysub to totsub;
	%end; *If totsub >0;
	ods tagsets.excelxp close;
	ods listing;
	%mend out;
	%out();

/**********************************************************************
	Output inputs for R-code sequential monitoring to .csv file
***********************************************************************/
	%macro r_seq();
  
	data crude matched;

		set output.estimates_&requestid. (keep = monitoringperiod analysis n_case_&e1 n_case_&e0); 

		if analysis = "Summary Estimate: stratified by dpidsiteid" then do;
			crude_total_cases = n_case_&e1. + n_case_&e0.;
			crude_n_case_&e1. = n_case_&e1.;
			crude_n_case_&e0. = n_case_&e0.;
			output crude;
		end;

		if analysis = "Summary Estimate: stratified by dpidsiteid, &ps.matchid1_&caliper." then do;
			matched1to1_total_cases = n_case_&e1. + n_case_&e0.;
			matched1to1_n_case_&e1. = n_case_&e1.;
			matched1to1_n_case_&e0. = n_case_&e0.;
			output matched;
		end;
	
	run;

	data output.r_input_&requestid.;
		merge crude (keep = monitoringperiod crude_total_cases crude_n_case_&e1. crude_n_case_&e0.)
			  matched (keep = monitoringperiod matched1to1_total_cases matched1to1_n_case_&e1. matched1to1_n_case_&e0.);
		by monitoringperiod;
	run; 

	%let filename = &REQUESTID._&numAgg.DP_Period&first.toPeriod&period._r_sequential_input;

	ods csv file = "&output.&filename..csv";
	proc print data = output.r_input_&requestid. noobs; 
	run;
	ods csv close;

	%mend;
	%r_seq();


%mend;
/********************************************************************************
   5. Specify which monitoring periods to analyze								
 																				
   Note: By default, the macro will start at the first (first = 1) 				
 		 monitoring period and end at the first monitoring period (period = 1)  
		 with no additional subgroup analyses. 
	
		 Enter different values in the %sequential call below for analyses of 	
 		 different monitoring periods with/without additional subgroup analyses
	
		 Select which propensity score and matching caliper to use for adjusted analyses. 
		 Default is propensity score using demographics, predefined, and hdPS at caliper = .025.

	Examples:																		
    %sequential(first = 1, period = 12, subgroup = , PS = predefined, caliper = 2);							
	%sequential(first = 5, period = 7, subgroup = %quote(covar1, covproc8), PS = hdpspredefined, caliper = 1);	
/********************************************************************************/

	%sequential(first = 1, period = 1, subgroup = %quote(age_cat), PS = , caliper = );
